package com.winhc.bigdata.spark.ng.utils

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * @author: XuJiakai
 * @date: 2021/2/24 17:32
 */

case class field_handle(field_name: String, handle: String)

case class export_2_es_args(tn: String, out_fields: Seq[String], handles: Seq[field_handle] = null)

case class export_company_index_2_es(s: SparkSession,
                                     export_args: export_2_es_args
                                    ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s

  private val target_tab = s"winhc_ng.out_index_es_${export_args.tn}"


  private val cols = (export_args.out_fields :+ "deleted").distinct

  init()

  private def init(): Unit = {
    if (spark.catalog.tableExists(target_tab)) {
      val source_tab_cols = getColumns(target_tab).diff(Seq("ds"))
      if (source_tab_cols.diff(cols).nonEmpty || cols.diff(source_tab_cols).nonEmpty) {
        sql(
          s"""
             |DROP TABLE IF EXISTS $target_tab
             |""".stripMargin)
      }
    }

    if (!spark.catalog.tableExists(target_tab)) {
      sql(
        s"""
           |CREATE TABLE IF NOT EXISTS $target_tab
           |(
           |    ${cols.map(f => s"${f} STRING COMMENT '${f}'").mkString(",")}
           |)
           |COMMENT 'out index es company,create by ${BaseUtil.nowDate(pattern = "yyyy-MM-dd HH:mm:ss")}'
           |PARTITIONED BY (ds STRING COMMENT '分区')
           |LIFECYCLE 15
           |""".stripMargin)
    }


    def parse_double(v: String): String = {
      if (StringUtils.isEmpty(v)) {
        return null
      }
      val v_2 = v.trim
      try {
        v_2.toDouble
        v_2
      } catch {
        case ex: Exception => {
          null
        }
      }
    }

    spark.udf.register("parse_double", parse_double _)

  }

  private def get_all_tab(): Unit = {
    sql(
      s"""
         |SELECT  *
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds DESC ) AS num
         |            FROM    (
         |                        SELECT  *
         |                        FROM    winhc_ng.ads_${export_args.tn}
         |                        WHERE   ds > 0
         |                        UNION ALL
         |                        SELECT  *
         |                        FROM    winhc_ng.inc_ads_${export_args.tn}
         |                        WHERE   ds > 0
         |                    )
         |        )
         |WHERE   num = 1
         |""".stripMargin)
      .createTempView("export_all_tab")
  }

  private def get_inc_tab(start_ds: String): Unit = {
    sql(
      s"""
         |SELECT  *
         |FROM    (
         |            SELECT  *
         |                    ,ROW_NUMBER() OVER(PARTITION BY rowkey ORDER BY ds DESC ) AS num
         |            FROM    (
         |                        SELECT  *
         |                        FROM    winhc_ng.inc_ads_${export_args.tn}
         |                        WHERE   ds > $start_ds
         |                    )
         |        )
         |WHERE   num = 1
         |""".stripMargin).createTempView("export_all_tab")
  }


  def calc(): Unit = {
    val last_ds = getLastPartitionsOrElse(target_tab, null)

    if (last_ds == null) {
      // all
      get_all_tab()
    } else {
      // inc
      if (last_ds.equals(BaseUtil.getYesterday())) {
        sql(
          s"""
             |ALTER TABLE $target_tab DROP IF EXISTS PARTITION(ds='${last_ds}')
             |""".stripMargin)
        calc()
        return
      }

      get_inc_tab(last_ds)
    }

    val target_ds = BaseUtil.getYesterday()
    val target_cols = getColumns(target_tab).diff(Seq("ds"))

    val map: Map[String, String] = if (export_args.handles != null)
      export_args.handles.map(f => (f.field_name, f.handle)).toMap
    else
      Map.empty

    sql(
      s"""
         |INSERT OVERWRITE TABLE $target_tab PARTITION(ds='$target_ds')
         |SELECT ${target_cols.map(f => s"${map.getOrElse(f, f)} as $f").mkString(",")}
         |FROM
         |    export_all_tab
         |""".stripMargin)
  }
}

object export_company_index_2_es {
  val as = Seq(

    export_2_es_args("company_court_open_announcement"
      , "rowkey,defendant_info,plaintiff_info,start_date,case_no,case_reason".split(","))

    , export_2_es_args("company_staff"
      , "rowkey,company_id,staff_name,staff_type,deleted".split(","))

    , export_2_es_args("company_tm"
      , "rowkey,keyno,reg_no,int_cls,tm_name,app_date,applicant_cn,applicant_en,status,app_year,logo,deleted".split(","))

    , export_2_es_args("company_icp"
      , "rowkey,company_id,web_baxh,site_url,review_time,web_name,domain,deleted".split(","))

    , export_2_es_args("company_app_info"
      , "rowkey,company_id,name,filter_name,icon,type,classes,deleted".split(","))

    , export_2_es_args("company_holder"
      , "rowkey,company_id,holder_name,holder_id,holder_type,percent,deleted".split(","))

    , export_2_es_args("company_dishonest_info"
      , "rowkey,keyno,name,case_no,court,gist_dd,reg_time,performance,pub_date,status,deleted".split(","))

    , export_2_es_args("company_zxr_restrict"
      , "rowkey,company_id,company_name,pid,person_name,sex,card_num,court_name,case_create_time,case_no,status,deleted".split(","))

    , export_2_es_args("company_equity_info"
      , "rowkey,related_company_name,related_company_id,reg_number,pledgor_info,equity_amount,pledgee_info,reg_date,state,deleted".split(","))

    , export_2_es_args("company_land_mortgage"
      , "rowkey,mortgagor_company_id,mortgagee_company_id,mortgagor,mortgagee,use_for,area,evaluate_amount,mortgage_amount,start_date,deleted".split(","))

    , export_2_es_args("company_judicial_assistance"
      , "rowkey,company_name,company_id,fz_publicity_date,executed_person_id,executed_person,equity_amount,execute_notice_num,executive_court,type_state,deleted".split(","))

    , export_2_es_args("company_public_announcement"
      , "rowkey,company_id,bill_type,bill_num,bill_amt,apply_name,publish_date,court,deleted".split(","))

    , export_2_es_args("company_abnormal_info"
      , "rowkey,company_id,company_name,put_reason,put_date,put_department,remove_reason,remove_date,remove_department,deleted".split(","))

    , export_2_es_args("company_illegal_info"
      , "rowkey,company_id,company_name,put_reason,put_date,put_department,remove_reason,remove_date,remove_department,deleted".split(","))

    , export_2_es_args("zxr_evaluate_results"
      , "rowkey,keyno,name,court_name,case_no,asset_type,publish_time,reference_price,money,deleted".split(","))

    , export_2_es_args("zxr_evaluate"
      , "rowkey,keyno,name,court_name,case_no,asset_type,asset_name,deleted".split(","))

    , export_2_es_args("restrictions_on_exit"
      , "rowkey,limited_person_pid,case_no,limited_person,executed_person_keyno,executed_person_name,executed_address,executed_amount,deleted".split(","))


    , export_2_es_args("company_court_announcement"
      , "rowkey,announcement_type,case_no,court_name,plaintiff_info,litigant_info,publish_date,deleted".split(","))
    , export_2_es_args("company_send_announcement"
      , "rowkey,defendant_info,plaintiff_info,court,start_date,title,deleted".split(","))
    , export_2_es_args("company_court_register"
      , "rowkey,case_no,case_reason,defendant_info,plaintiff_info,filing_date,deleted".split(","))

    , export_2_es_args("company_zxr_final_case"
      , "rowkey,name,keyno,court_name,case_final_time,case_create_time,case_no,exec_amount,no_exec_amount,status,deleted".split(",")
      , handles = Seq(field_handle(field_name = "exec_amount", handle = "parse_double(exec_amount)"),field_handle(field_name = "no_exec_amount", handle = "parse_double(no_exec_amount)")))


    , export_2_es_args("company_tax_contravention"
      , "rowkey,company_id,company_name,case_type,case_info,department,publish_time,deleted".split(","))
    , export_2_es_args("company_own_tax"
      , "rowkey,company_id,company_name,legal_name,tax_category,own_tax_amount,publish_date,tax_balance,new_tax_balance,deleted".split(","))
    , export_2_es_args("company_check_info"
      , "rowkey,company_id,company_name,check_org,check_type,check_date,check_result,deleted".split(","))
    , export_2_es_args("company_mortgage_info"
      , "rowkey,company_id,company_name,reg_num,reg_date,publish_date,reg_department,type,deleted".split(","))
    , export_2_es_args("company_double_random_check_info"
      , "rowkey,company_id,company_name,check_plan_num,check_plan_name,check_task_num,check_task_name,check_type,check_department,check_date,deleted".split(","))


    , export_2_es_args("company_zxr"
      , "rowkey,name,keyno,card,court,case_create_time,case_no,exec_money,status,deleted".split(",")
      , handles = Seq(field_handle(field_name = "exec_money", handle = "parse_double(exec_money)")))

    , export_2_es_args("bankruptcy_open_case"
      , "rowkey,case_no,case_type,agency_court,applicant,applicant_info,respondent,respondent_info,public_date,deleted".split(","))
    , export_2_es_args("auction_tracking"
      , "rowkey,auction_items_id,company_info,case_no,auction_title,initial_price,start_time,apply_count,url,deleted".split(","))
    , export_2_es_args("company_punishment_info"
      , "rowkey,company_id,company_name,person_name,pid,type,department_name,decision_date,publish_date,deleted".split(","))
    , export_2_es_args("company_punishment_info_creditchina"
      , "rowkey,company_id,company_name,punish_number,punish_name,person_name,pid,decision_date,status,deleted".split(","))
    , export_2_es_args("company_brief_cancel_announcement"
      , "rowkey,company_id,deleted".split(","))
  )


  def run(spark: SparkSession, tn: String): Unit = {
    export_company_index_2_es(spark, export_args = as.find(p => p.tn.equals(tn)).get)
      .calc()
  }


  def main(args: Array[String]): Unit = {
    val Array(tn) = args
    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> "winhc_ng",
      "spark.debug.maxToStringFields" -> "200",
      "spark.hadoop.odps.spark.local.partition.amt" -> "10"
    )
    val spark = SparkUtils.InitEnv(this.getClass.getName, config)
    run(spark, tn)

    spark.stop()
  }
}
